@@ -4,10 +4,14 @@ from .discrete_field import DiscreteField
 
 
 class ElementHcurl(Element):
-    """Note: only 3D support. Piola transformation
-    is different in 2D."""
+    """:math:`H(curl)`-conforming basis defined through a reference element.
+
+    Supports 3D meshes only.
+
+    """
 
     def orient(self, mapping, i, tind=None):
+        """Orientation based on the edge node indexing."""
         if tind is not None:
             # TODO fix
             raise NotImplementedError("TODO: fix tind support in ElementHcurl")
@@ -16,6 +20,7 @@ class ElementHcurl(Element):
         return 1 - 2 * (mapping.mesh.t[t1] > mapping.mesh.t[t2])
 
     def gbasis(self, mapping, X, i, tind=None):
+        """Contravariant Piola transformation."""
         phi, dphi = self.lbasis(X, i)
         DF = mapping.DF(X, tind)
         invDF = mapping.invDF(X, tind)
