@@ -1,4 +1,5 @@
 from .element_hex0 import ElementHex0  # noqa
-from .element_hex1 import ElementHex1  # noqa
+from .element_hex1 import ElementHex1, ElementHex1DG  # noqa
 from .element_hex_s2 import ElementHexS2  # noqa
 from .element_hex2 import ElementHex2  # noqa
+from .element_hex_dg import ElementHexDG  # noqa
