@@ -1,5 +1,5 @@
 from .element_quad0 import ElementQuad0  # noqa
-from .element_quad1 import ElementQuad1  # noqa
+from .element_quad1 import ElementQuad1, ElementQuad1DG  # noqa
 from .element_quad2 import ElementQuad2  # noqa
 from .element_quad_s2 import ElementQuadS2  # noqa
 from .element_quad_dg import ElementQuadDG  # noqa
