@@ -1,11 +1,17 @@
-from .element_tri_p1 import ElementTriP1  # noqa
+from .element_tri_p1 import ElementTriP1, ElementTriP1DG  # noqa
 from .element_tri_p2 import ElementTriP2  # noqa
+from .element_tri_p3 import ElementTriP3  # noqa
+from .element_tri_p4 import ElementTriP4  # noqa
 from .element_tri_dg import ElementTriDG  # noqa
 from .element_tri_cr import ElementTriCR  # noqa
 from .element_tri_p0 import ElementTriP0  # noqa
 from .element_tri_rt0 import ElementTriRT0  # noqa
+from .element_tri_bdm1 import ElementTriBDM1  # noqa
 from .element_tri_mini import ElementTriMini  # noqa
 from .element_tri_morley import ElementTriMorley  # noqa
+from .element_tri_15param_plate import ElementTri15ParamPlate  # noqa
 from .element_tri_argyris import ElementTriArgyris  # noqa
 from .element_tri_hermite import ElementTriHermite  # noqa
-from .element_tri_ccr import ElementTriCCR # noqa
+from .element_tri_ccr import ElementTriCCR  # noqa
+from .element_tri_skeleton_p0 import ElementTriSkeletonP0  # noqa
+from .element_tri_skeleton_p1 import ElementTriSkeletonP1  # noqa
