@@ -30,6 +30,9 @@ def curl(u: DiscreteField):
     """Curl."""
     if u.curl is not None:
         return u.curl
+    elif u.grad is not None:
+        if u.grad.shape[0] == 2:
+            return np.array([u.grad[1], -u.grad[0]])
     raise NotImplementedError
 
 
@@ -49,16 +52,19 @@ def sym_grad(u: DiscreteField):
     return .5 * (u.grad + transpose(u.grad))
 
 
-def dd(u):
+def dd(u: DiscreteField):
+    """Hessian (if available)."""
     return u.hess
 
 
 def ddd(u):
-    return u.hod[0]
+    """Third derivative (if available)."""
+    return u.grad3
 
 
 def dddd(u):
-    return u.hod[1]
+    """Fourth derivative (if available)."""
+    return u.grad4
 
 
 def dot(u: FieldOrArray, v: FieldOrArray):
@@ -99,18 +105,40 @@ def eye(w, n):
                      for j in range(n)])
 
 
+def identity(w, N=None):
+    """Create identity matrix."""
+    if isinstance(w, DiscreteField):
+        proto = w.value
+    elif isinstance(w, ndarray):
+        proto = w
+    else:
+        raise NotImplementedError
+
+    if N is None:
+        if len(proto.shape) > 2:
+            N = proto.shape[-3]
+        else:
+            raise ValueError("Cannot deduce the size of the identity matrix. "
+                             "Give an explicit keyword argument N.")
+
+    return eye(np.ones(proto.shape[-2:]), N)
+
+
 def det(A):
     """
-    Determinant of an array `A` over trailing axis (if any)
+    Determinant of an array `A` over trailing axis (if any).
+
     Parameters
     ----------
     A : (N, N,...) numpy.ndarray
         N = 2 or 3
         Input array whose determinant is to be computed
+
     Returns
     -------
     det : (...) numpy.ndarray
         Determinant of `A`.
+
     """
     detA = zeros_like(A[0, 0])
     if A.shape[0] == 3:
@@ -126,17 +154,19 @@ def det(A):
 
 
 def inv(A):
-    """
-    Inverse of an array `A` over trailing axis (if any)
+    """Inverse of an array `A` over trailing axis (if any)
+
     Parameters
     ----------
     A : (N, N,...) numpy.ndarray
         N = 2 or 3
         Input array whose inverse is to be computed
+
     Returns
     -------
     Ainv : (N, N,...) numpy.ndarray
         Inverse of `A`.
+
     """
     invA = zeros_like(A)
     detA = det(A)
