@@ -1,4 +1,3 @@
-from collections import namedtuple
 from dataclasses import dataclass, replace
 from typing import Callable, Dict, List, Optional, Tuple, Type, Union
 from warnings import warn
@@ -23,8 +22,7 @@ class Mesh:
     _boundaries: Optional[Dict[str, ndarray]] = None
     _subdomains: Optional[Dict[str, ndarray]] = None
     elem: Type[Element] = Element
-    affine: bool = False
-    validate: bool = False  # unused; for backwards compatibility
+    affine: bool = False  #: Use :class:`~skfem.mapping.MappingAffine`?
     # Some parts of the library, most notably the normal vector construction in
     # ElementGlobal._eval_dofs, assume that the element indices are ascending
     # because this leads to consistent normal vectors for both elements sharing
@@ -54,7 +52,9 @@ class Mesh:
 
     @property
     def bndelem(self):
-        return BOUNDARY_ELEMENT_MAP[self.elem]()
+        if self.elem in BOUNDARY_ELEMENT_MAP:
+            return BOUNDARY_ELEMENT_MAP[self.elem]()
+        return None
 
     @property
     def nelements(self):
@@ -139,30 +139,9 @@ class Mesh:
         ))[0]
         return edge_candidates[ix]
 
-    def define_boundary(self, name: str,
-                        test: Callable[[ndarray], ndarray],
-                        boundaries_only: bool = True):
-        """Define a named boundary via function handle.
-
-        Parameters
-        ----------
-        name
-            Name of the boundary.
-        test
-            A function which returns True for facet midpoints belonging to the
-            boundary.
-        boundaries_only
-            If True, include only facets on the boundary of the mesh.
-
-        """
-        warn("Mesh.define_boundary is deprecated and will be removed in the "
-             "next major release.", DeprecationWarning)
-        if self._boundaries is None:
-            self._boundaries = {}
-        self._boundaries[name] = self.facets_satisfying(test, boundaries_only)
-
     def with_boundaries(self,
-                        boundaries: Dict[str, Callable[[ndarray], ndarray]]):
+                        boundaries: Dict[str, Callable[[ndarray], ndarray]],
+                        boundaries_only: bool = True):
         """Return a copy of the mesh with named boundaries.
 
         Parameters
@@ -171,13 +150,15 @@ class Mesh:
             A dictionary of lambda functions with the names of the boundaries
             as keys.  The midpoint of the facet should return ``True`` for the
             corresponding lambda function if the facet belongs to the boundary.
+        boundaries_only
+            If ``True``, consider only facets on the boundary of the domain.
 
         """
         return replace(
             self,
             _boundaries={
                 **({} if self._boundaries is None else self._boundaries),
-                **{name: self.facets_satisfying(test, True)
+                **{name: self.facets_satisfying(test, boundaries_only)
                    for name, test in boundaries.items()}
             },
         )
@@ -204,6 +185,69 @@ class Mesh:
             }
         )
 
+    def _encode_point_data(self) -> Dict[str, List[ndarray]]:
+
+        subdomains = {} if self._subdomains is None else self._subdomains
+        boundaries = {} if self._boundaries is None else self._boundaries
+
+        def indicator(ix):
+            ind = np.zeros(self.nvertices)
+            ind[ix] = 1
+            return ind
+
+        return {
+            **{
+                f"skfem:s:{name}": indicator(np.unique(self.t[:, subdomain]
+                                                       .flatten()))
+                for name, subdomain in subdomains.items()
+            },
+            **{
+                f"skfem:b:{name}": indicator(np.unique(self.facets[:, boundary]
+                                                       .flatten()))
+                for name, boundary in boundaries.items()
+            },
+        }
+
+    def _encode_cell_data(self) -> Dict[str, List[ndarray]]:
+
+        subdomains = {} if self._subdomains is None else self._subdomains
+        boundaries = {} if self._boundaries is None else self._boundaries
+
+        return {
+            **{
+                f"skfem:s:{name}": [
+                    np.isin(np.arange(self.t.shape[1]), subdomain).astype(int)
+                ]
+                for name, subdomain in subdomains.items()
+            },
+            **{
+                f"skfem:b:{name}": [
+                    ((1 << np.arange(self.t2f.shape[0]))
+                     @ np.isin(self.t2f, boundary)).astype(int)
+                ]
+                for name, boundary in boundaries.items()
+            },
+        }
+
+    def _decode_cell_data(self, cell_data: Dict[str, List[ndarray]]):
+
+        subdomains = {}
+        boundaries = {}
+
+        for name, data in cell_data.items():
+            subnames = name.split(":")
+            if subnames[0] != "skfem":
+                continue
+            if subnames[1] == "s":
+                subdomains[subnames[2]] = np.nonzero(data[0])[0]
+            elif subnames[1] == "b":
+                boundaries[subnames[2]] = self.t2f[
+                    (1 << np.arange(self.t2f.shape[0]))[:, None]
+                    & data[0].astype(np.int64) > 0
+                ]
+
+        return boundaries, subdomains
+
     def boundary_nodes(self) -> ndarray:
         """Return an array of boundary node indices."""
         return np.unique(self.facets[:, self.boundary_facets()])
@@ -301,21 +345,8 @@ class Mesh:
             if self.affine:
                 self._cached_mapping = MappingAffine(self)
             else:
-                # TODO make MappingIsoparametric compatible with self
-                FakeMesh = namedtuple(
-                    'FakeMesh',
-                    ['p', 't', 'facets', 't2f', 'f2t', 'dim']
-                )
-                fakemesh = FakeMesh(
-                    self.doflocs,
-                    self.dofs.element_dofs,
-                    self.facets,
-                    self.t2f,
-                    self.f2t,
-                    lambda: self.dim(),
-                )
                 self._cached_mapping = MappingIsoparametric(
-                    fakemesh,
+                    self,
                     self.elem(),
                     self.bndelem,
                 )
@@ -380,26 +411,81 @@ class Mesh:
                 warn("Transforming over 1000 elements to C_CONTIGUOUS.")
             self.t = np.ascontiguousarray(self.t)
 
+    def __rmatmul__(self, other):
+        out = self.__matmul__(other)
+        return out[1:] + out[0:1]
+
+    def __matmul__(self, other):
+        """Join meshes with different types; return a list of meshes."""
+        cls = type(self)
+        if isinstance(other, Mesh):
+            other = [other]
+        if isinstance(other, list):
+            p = np.hstack((self.p,) + tuple([mesh.p for mesh in other]))
+            pT = np.ascontiguousarray(p.T)
+            _, ixa, ixb = np.unique(pT.view([('', pT.dtype)] * pT.shape[1]),
+                                    return_index=True, return_inverse=True)
+            p = p[:, ixa]
+            return [
+                cls(p, ixb[self.t]),
+                *[type(m)(p, ixb[m.t + self.p.shape[1]])
+                  for i, m in enumerate(other)],
+            ]
+        raise NotImplementedError
+
+    def is_valid(self) -> bool:
+        """Perform some mesh validation checks."""
+        # check that there are no duplicate points
+        tmp = np.ascontiguousarray(self.p.T)
+        if self.p.shape[1] != np.unique(tmp.view([('', tmp.dtype)]
+                                                 * tmp.shape[1])).shape[0]:
+            warn("Mesh contains duplicate vertices.")
+            return False
+
+        # check that all points are at least in some element
+        if len(np.setdiff1d(np.arange(self.p.shape[1]),
+                            np.unique(self.t))) > 0:
+            warn("Mesh contains a vertex not belonging to any element.")
+            return False
+
+        return True
+
     def __add__(self, other):
         """Join two meshes."""
-        if not isinstance(other, type(self)):
+        cls = type(self)
+        if not isinstance(other, cls):
             raise TypeError("Can only join meshes with same type.")
-        p = np.hstack((self.p, other.p))
+        p = np.hstack((self.p.round(decimals=8),
+                       other.p.round(decimals=8)))
         t = np.hstack((self.t, other.t + self.p.shape[1]))
         tmp = np.ascontiguousarray(p.T)
         tmp, ixa, ixb = np.unique(tmp.view([('', tmp.dtype)] * tmp.shape[1]),
                                   return_index=True, return_inverse=True)
         p = p[:, ixa]
         t = ixb[t]
-        cls = type(self)
         return cls(p, t)
 
     def __repr__(self):
-        return "{} mesh with {} vertices and {} elements.".format(
-            self.elem.refdom.name,
-            self.nvertices,
-            self.nelements,
-        )
+        rep = ""
+        rep += "<skfem {} object>\n".format(type(self).__name__)
+        rep += "  Number of elements: {}\n".format(self.nelements)
+        rep += "  Number of vertices: {}\n".format(self.nvertices)
+        rep += "  Number of nodes: {}".format(self.p.shape[1])
+        if self.subdomains is not None:
+            rep += "\n  Named subdomains [# elements]: {}".format(
+                ', '.join(
+                    map(lambda k: '{} [{}]'.format(k, len(self.subdomains[k])),
+                        list(self.subdomains.keys()))
+                )
+            )
+        if self.boundaries is not None:
+            rep += "\n  Named boundaries [# facets]: {}".format(
+                ', '.join(
+                    map(lambda k: '{} [{}]'.format(k, len(self.boundaries[k])),
+                        list(self.boundaries.keys()))
+                )
+            )
+        return rep
 
     def __str__(self):
         return self.__repr__()
@@ -407,6 +493,7 @@ class Mesh:
     def save(self,
              filename: str,
              point_data: Optional[Dict[str, ndarray]] = None,
+             cell_data: Optional[Dict[str, ndarray]] = None,
              **kwargs) -> None:
         """Export the mesh and fields using meshio.
 
@@ -417,67 +504,58 @@ class Mesh:
             e.g. .msh, .vtk, .xdmf
         point_data
             Data related to the vertices of the mesh.
+        cell_data
+            Data related to the elements of the mesh.
 
         """
         from skfem.io.meshio import to_file
-        return to_file(self, filename, point_data, **kwargs)
+        return to_file(self,
+                       filename,
+                       point_data,
+                       cell_data,
+                       **kwargs)
 
     @classmethod
-    def load(cls, filename: str):
+    def load(cls,
+             filename: str,
+             out: Optional[List[str]] = None,
+             **kwargs):
         """Load a mesh using meshio.
 
         Parameters
         ----------
         filename
             The filename of the mesh file.
+        out
+            Optional list of ``meshio.Mesh`` attribute names, overwrite with
+            the corresponding data.  E.g., ``['point_data', 'cell_data']``.
 
         """
         from skfem.io.meshio import from_file
-        return from_file(filename)
+        return from_file(filename,
+                         out,
+                         **kwargs)
 
     @classmethod
-    def from_dict(cls, data):
-        """For backwards compatibility."""
-        if 'p' not in data or 't' not in data:
-            raise ValueError("Dictionary must contain keys 'p' and 't'.")
-        else:
-            data['p'] = np.ascontiguousarray(np.array(data['p']).T)
-            data['t'] = np.ascontiguousarray(np.array(data['t']).T)
-        if 'boundaries' in data and data['boundaries'] is not None:
-            data['boundaries'] = {k: np.array(v)
-                                  for k, v in data['boundaries'].items()}
-        if 'subdomains' in data and data['subdomains'] is not None:
-            data['subdomains'] = {k: np.array(v)
-                                  for k, v in data['subdomains'].items()}
-        data['doflocs'] = data.pop('p')
-        data['_subdomains'] = data.pop('subdomains')
-        data['_boundaries'] = data.pop('boundaries')
-        return cls(**data)
+    def from_dict(cls, d):
+        from skfem.io.json import from_dict
+        return from_dict(cls, d)
 
-    def to_dict(self) -> Dict[str, Optional[Dict[str, List[float]]]]:
-        """For backwards compatibility."""
-        boundaries: Optional[Dict[str, List[float]]] = None
-        subdomains: Optional[Dict[str, List[float]]] = None
-        if self.boundaries is not None:
-            boundaries = {k: v.tolist() for k, v in self.boundaries.items()}
-        if self.subdomains is not None:
-            subdomains = {k: v.tolist() for k, v in self.subdomains.items()}
-        return {
-            'p': self.p.T.tolist(),
-            't': self.t.T.tolist(),
-            'boundaries': boundaries,
-            'subdomains': subdomains,
-        }
+    def to_dict(self):
+        from skfem.io.json import to_dict
+        return to_dict(self)
 
     @classmethod
-    def from_mesh(cls, mesh):
-        """Reuse an existing mesh by adding nodes.
+    def from_mesh(cls, mesh, t: Optional[ndarray] = None):
+        """Reuse an existing mesh by adding the higher order nodes.
 
         Parameters
         ----------
         mesh
             The mesh used in the initialization.  Connectivity of the new mesh
-            will match ``mesh.t``.
+            will match ``mesh.t`` unless ``t`` is given
+        t
+            Optionally specify new connectivity for the resulting mesh.
 
         """
         from skfem.assembly import Dofs
@@ -488,6 +566,8 @@ class Mesh:
         locs = mapping.F(nelem.doflocs.T)
         doflocs = np.zeros((locs.shape[0], dofs.N))
 
+        assert dofs.element_dofs is not None
+
         # match mapped dofs and global dof numbering
         for itr in range(locs.shape[0]):
             for jtr in range(dofs.element_dofs.shape[0]):
@@ -495,7 +575,7 @@ class Mesh:
 
         return cls(
             doflocs=doflocs,
-            t=mesh.t,
+            t=t if t is not None else mesh.t,
         )
 
     @classmethod
@@ -640,6 +720,8 @@ class Mesh:
     @staticmethod
     def build_entities(t, indices, sort=True):
         """Build low dimensional topological entities."""
+        if indices is None:
+            return None, None
         indexing = np.hstack(tuple([t[ix] for ix in indices]))
         sorted_indexing = np.sort(indexing, axis=0)
 
@@ -667,6 +749,7 @@ class Mesh:
         inverse = np.zeros((2, np.max(mapping) + 1), dtype=np.int64)
         inverse[0, e_first] = tix[ix_first]
         inverse[1, e_last] = tix[ix_last]
+
         inverse[1, np.nonzero(inverse[0] == inverse[1])[0]] = -1
 
         return inverse
@@ -713,3 +796,8 @@ class Mesh:
 
         """
         raise NotImplementedError
+
+    def draw(self, *args, **kwargs):
+        """Convenience wrapper for vedo."""
+        from skfem.visuals.vedo import draw
+        return draw(self, *args, **kwargs)
